@@ -38,19 +38,18 @@
           - multiplicity of TLVs (if given by the standard)
           - min sizes of strings used by the TLVs
         - conf.contribs['LLDP'].strict_mode_disable() -> disable strict mode
-        - strict mode = True => conf.debug_dissector = True
 
 """
 from scapy.config import conf
-from scapy.error import log_runtime, Scapy_Exception
+from scapy.error import Scapy_Exception
 from scapy.layers.l2 import Ether, Dot1Q
 from scapy.fields import MACField, IPField, BitField, \
     StrLenField, ByteEnumField, BitEnumField, \
     EnumField, ThreeBytesField, BitFieldLenField, \
-    ShortField, XStrLenField, ByteField
+    ShortField, XStrLenField, ByteField, ConditionalField, \
+    MultipleTypeField
 from scapy.packet import Packet, bind_layers
 from scapy.modules.six.moves import range
-import scapy.modules.six as six
 from scapy.data import ETHER_TYPES
 from scapy.compat import orb
 
@@ -298,37 +297,16 @@ class LLDPDU(Packet):
         return super(LLDPDU, self).do_build()
 
 
-class _LLDPidField(StrLenField):
-    """Class that selects the type of the ID field depending
-    on the type of `subtype`"""
-    __slots__ = StrLenField.__slots__ + ["subtypes_dict"]
-
-    def __init__(self, name, default, subtypes_dict, *args, **kargs):
-        self.subtypes_dict = subtypes_dict
-        super(_LLDPidField, self).__init__(name, default, *args, **kargs)
-
-    def m2i(self, pkt, x):
-        cls = self.subtypes_dict.get(pkt.subtype, StrLenField)
-        try:
-            return (cls.m2i.__func__ if six.PY2 else cls.m2i)(self, pkt, x)
-        except Exception:
-            log_runtime.exception("Failed to dissect " + self.name + " ! ")
-            return StrLenField.m2i(self, pkt, x)
-
-    def i2m(self, pkt, x):
-        cls = self.subtypes_dict.get(pkt.subtype, StrLenField)
-        try:
-            return (cls.i2m.__func__ if six.PY2 else cls.i2m)(self, pkt, x)
-        except Exception:
-            log_runtime.exception("Failed to build " + self.name + " ! ")
-            return StrLenField.i2m(self, pkt, x)
-
-
 def _ldp_id_adjustlen(pkt, x):
     """Return the length of the `id` field,
     according to its real encoded type"""
     f, v = pkt.getfield_and_val('id')
-    return len(_LLDPidField.i2m(f, pkt, v)) + 1
+    length = f.i2len(pkt, v) + 1
+    if (isinstance(pkt, LLDPDUPortID) and pkt.subtype == 0x4) or \
+            (isinstance(pkt, LLDPDUChassisID) and pkt.subtype == 0x5):
+        # Take the ConditionalField into account
+        length += 1
+    return length
 
 
 class LLDPDUChassisID(LLDPDU):
@@ -347,11 +325,6 @@ class LLDPDUChassisID(LLDPDU):
         range(0x08, 0xff): 'reserved'
     }
 
-    LLDP_CHASSIS_ID_TLV_SUBTYPES_FIELDS = {
-        0x04: MACField,
-        0x05: IPField,
-    }
-
     SUBTYPE_RESERVED = 0x00
     SUBTYPE_CHASSIS_COMPONENT = 0x01
     SUBTYPE_INTERFACE_ALIAS = 0x02
@@ -366,7 +339,21 @@ class LLDPDUChassisID(LLDPDU):
         BitFieldLenField('_length', None, 9, length_of='id',
                          adjust=lambda pkt, x: _ldp_id_adjustlen(pkt, x)),
         ByteEnumField('subtype', 0x00, LLDP_CHASSIS_ID_TLV_SUBTYPES),
-        _LLDPidField('id', '', LLDP_CHASSIS_ID_TLV_SUBTYPES_FIELDS, length_from=lambda pkt: pkt._length - 1)  # noqa: E501
+        ConditionalField(
+            ByteField('family', 0),
+            lambda pkt: pkt.subtype == 0x05
+        ),
+        MultipleTypeField([
+            (
+                MACField('id', None),
+                lambda pkt: pkt.subtype == 0x04
+            ),
+            (
+                IPField('id', None),
+                lambda pkt: pkt.subtype == 0x05
+            ),
+        ], StrLenField('id', '', length_from=lambda pkt: pkt._length - 1)
+        )
     ]
 
     def _check(self):
@@ -393,11 +380,6 @@ class LLDPDUPortID(LLDPDU):
         range(0x08, 0xff): 'reserved'
     }
 
-    LLDP_PORT_ID_TLV_SUBTYPES_FIELDS = {
-        0x03: MACField,
-        0x04: IPField,
-    }
-
     SUBTYPE_RESERVED = 0x00
     SUBTYPE_INTERFACE_ALIAS = 0x01
     SUBTYPE_PORT_COMPONENT = 0x02
@@ -412,7 +394,21 @@ class LLDPDUPortID(LLDPDU):
         BitFieldLenField('_length', None, 9, length_of='id',
                          adjust=lambda pkt, x: _ldp_id_adjustlen(pkt, x)),
         ByteEnumField('subtype', 0x00, LLDP_PORT_ID_TLV_SUBTYPES),
-        _LLDPidField('id', '', LLDP_PORT_ID_TLV_SUBTYPES_FIELDS, length_from=lambda pkt: pkt._length - 1)  # noqa: E501
+        ConditionalField(
+            ByteField('family', 0),
+            lambda pkt: pkt.subtype == 0x04
+        ),
+        MultipleTypeField([
+            (
+                MACField('id', None),
+                lambda pkt: pkt.subtype == 0x03
+            ),
+            (
+                IPField('id', None),
+                lambda pkt: pkt.subtype == 0x04
+            ),
+        ], StrLenField('id', '', length_from=lambda pkt: pkt._length - 1)
+        )
     ]
 
     def _check(self):
@@ -738,14 +734,12 @@ class LLDPConfiguration(object):
         enable strict mode and dissector debugging
         """
         self._strict_mode = True
-        conf.debug_dissector = True
 
     def strict_mode_disable(self):
         """
         disable strict mode and dissector debugging
         """
         self._strict_mode = False
-        conf.debug_dissector = False
 
     def strict_mode(self):
         """
