@@ -1,17 +1,21 @@
-## This file is part of Scapy
-## Copyright (C) 2017 Maxence Tury
-## This program is published under a GPLv2 license
+# This file is part of Scapy
+# Copyright (C) 2017 Maxence Tury
+# This program is published under a GPLv2 license
 
 """
 SSLv2 handshake fields & logic.
 """
 
-import math
+import struct
 
+from scapy.config import conf
 from scapy.error import log_runtime, warning
-from scapy.fields import *
-from scapy.packet import Packet, Raw, Padding
-from scapy.layers.tls.cert import Cert, PrivKey, PubKey
+from scapy.utils import randstring
+from scapy.fields import ByteEnumField, ByteField, EnumField, FieldLenField, \
+    ShortEnumField, StrLenField, XStrField, XStrLenField
+
+from scapy.packet import Padding
+from scapy.layers.tls.cert import Cert
 from scapy.layers.tls.basefields import _tls_version, _TLSVersionField
 from scapy.layers.tls.handshake import _CipherSuitesField
 from scapy.layers.tls.keyexchange import _TLSSignatureField, _TLSSignature
@@ -19,21 +23,19 @@ from scapy.layers.tls.session import (_GenericTLSSessionInheritance,
                                       readConnState, writeConnState)
 from scapy.layers.tls.crypto.suites import (_tls_cipher_suites,
                                             _tls_cipher_suites_cls,
-                                            _GenericCipherSuite,
-                                            _GenericCipherSuiteMetaclass,
                                             get_usable_ciphersuites,
                                             SSL_CK_DES_192_EDE3_CBC_WITH_MD5)
 
 
 ###############################################################################
-### Generic SSLv2 Handshake message                                         ###
+#   Generic SSLv2 Handshake message                                           #
 ###############################################################################
 
-_sslv2_handshake_type = { 0: "error",                1: "client_hello",
-                          2: "client_master_key",    3: "client_finished",
-                          4: "server_hello",         5: "server_verify",
-                          6: "server_finished",      7: "request_certificate",
-                          8: "client_certificate" }
+_sslv2_handshake_type = {0: "error", 1: "client_hello",
+                         2: "client_master_key", 3: "client_finished",
+                         4: "server_hello", 5: "server_verify",
+                         6: "server_finished", 7: "request_certificate",
+                         8: "client_certificate"}
 
 
 class _SSLv2Handshake(_GenericTLSSessionInheritance):
@@ -42,7 +44,7 @@ class _SSLv2Handshake(_GenericTLSSessionInheritance):
     Also used as a fallback for unknown TLS Handshake packets.
     """
     name = "SSLv2 Handshake Generic message"
-    fields_desc = [ ByteEnumField("msgtype", None, _sslv2_handshake_type) ]
+    fields_desc = [ByteEnumField("msgtype", None, _sslv2_handshake_type)]
 
     def guess_payload_class(self, p):
         return Padding
@@ -56,23 +58,24 @@ class _SSLv2Handshake(_GenericTLSSessionInheritance):
 
 
 ###############################################################################
-### Error                                                                   ###
+#   Error                                                                     #
 ###############################################################################
 
-_tls_error_code = { 1: "no_cipher",         2: "no_certificate",
-                    4: "bad_certificate",   6: "unsupported_certificate_type" }
+_tls_error_code = {1: "no_cipher", 2: "no_certificate",
+                   4: "bad_certificate", 6: "unsupported_certificate_type"}
+
 
 class SSLv2Error(_SSLv2Handshake):
     """
     SSLv2 Error.
     """
     name = "SSLv2 Handshake - Error"
-    fields_desc = [ ByteEnumField("msgtype", 0, _sslv2_handshake_type),
-                    ShortEnumField("code", None, _tls_error_code) ]
+    fields_desc = [ByteEnumField("msgtype", 0, _sslv2_handshake_type),
+                   ShortEnumField("code", None, _tls_error_code)]
 
 
 ###############################################################################
-### ClientHello                                                             ###
+#   ClientHello                                                               #
 ###############################################################################
 
 class _SSLv2CipherSuitesField(_CipherSuitesField):
@@ -101,24 +104,24 @@ class SSLv2ClientHello(_SSLv2Handshake):
     SSLv2 ClientHello.
     """
     name = "SSLv2 Handshake - Client Hello"
-    fields_desc = [ ByteEnumField("msgtype", 1, _sslv2_handshake_type),
-                    _TLSVersionField("version", 0x0002, _tls_version),
+    fields_desc = [ByteEnumField("msgtype", 1, _sslv2_handshake_type),
+                   _TLSVersionField("version", 0x0002, _tls_version),
 
-                    FieldLenField("cipherslen", None, fmt="!H",
-                                  length_of="ciphers"),
-                    FieldLenField("sidlen", None, fmt="!H",
-                                  length_of="sid"),
-                    FieldLenField("challengelen", None, fmt="!H",
-                                  length_of="challenge"),
+                   FieldLenField("cipherslen", None, fmt="!H",
+                                 length_of="ciphers"),
+                   FieldLenField("sidlen", None, fmt="!H",
+                                 length_of="sid"),
+                   FieldLenField("challengelen", None, fmt="!H",
+                                 length_of="challenge"),
 
-                    XStrLenField("sid", b"",
-                                 length_from=lambda pkt:pkt.sidlen),
-                    _SSLv2CipherSuitesField("ciphers",
-                                      [SSL_CK_DES_192_EDE3_CBC_WITH_MD5],
-                                      _tls_cipher_suites,
-                                      length_from=lambda pkt: pkt.cipherslen),
-                    XStrLenField("challenge", b"",
-                                 length_from=lambda pkt:pkt.challengelen) ]
+                   XStrLenField("sid", b"",
+                                length_from=lambda pkt:pkt.sidlen),
+                   _SSLv2CipherSuitesField("ciphers",
+                                           [SSL_CK_DES_192_EDE3_CBC_WITH_MD5],
+                                           _tls_cipher_suites,
+                                           length_from=lambda pkt: pkt.cipherslen),  # noqa: E501
+                   XStrLenField("challenge", b"",
+                                length_from=lambda pkt:pkt.challengelen)]
 
     def tls_session_update(self, msg_str):
         super(SSLv2ClientHello, self).tls_session_update(msg_str)
@@ -128,19 +131,21 @@ class SSLv2ClientHello(_SSLv2Handshake):
 
 
 ###############################################################################
-### ServerHello                                                             ###
+#   ServerHello                                                               #
 ###############################################################################
 
 class _SSLv2CertDataField(StrLenField):
     def getfield(self, pkt, s):
-        l = 0
+        tmp_len = 0
         if self.length_from is not None:
-            l = self.length_from(pkt)
+            tmp_len = self.length_from(pkt)
         try:
-            certdata = Cert(s[:l])
-        except:
-            certdata = s[:l]
-        return s[l:], certdata
+            certdata = Cert(s[:tmp_len])
+        except Exception:
+            if conf.debug_dissector:
+                raise
+            certdata = s[:tmp_len]
+        return s[tmp_len:], certdata
 
     def i2len(self, pkt, i):
         if isinstance(i, Cert):
@@ -158,25 +163,25 @@ class SSLv2ServerHello(_SSLv2Handshake):
     SSLv2 ServerHello.
     """
     name = "SSLv2 Handshake - Server Hello"
-    fields_desc = [ ByteEnumField("msgtype", 4, _sslv2_handshake_type),
+    fields_desc = [ByteEnumField("msgtype", 4, _sslv2_handshake_type),
 
-                    ByteField("sid_hit", 0),
-                    ByteEnumField("certtype", 1, {1: "x509_cert"}),
-                    _TLSVersionField("version", 0x0002, _tls_version),
+                   ByteField("sid_hit", 0),
+                   ByteEnumField("certtype", 1, {1: "x509_cert"}),
+                   _TLSVersionField("version", 0x0002, _tls_version),
 
-                    FieldLenField("certlen", None, fmt="!H",
-                                  length_of="cert"),
-                    FieldLenField("cipherslen", None, fmt="!H",
-                                  length_of="ciphers"),
-                    FieldLenField("connection_idlen", None, fmt="!H",
-                                  length_of="connection_id"),
+                   FieldLenField("certlen", None, fmt="!H",
+                                 length_of="cert"),
+                   FieldLenField("cipherslen", None, fmt="!H",
+                                 length_of="ciphers"),
+                   FieldLenField("connection_idlen", None, fmt="!H",
+                                 length_of="connection_id"),
 
-                    _SSLv2CertDataField("cert", b"",
-                                        length_from=lambda pkt: pkt.certlen),
-                    _SSLv2CipherSuitesField("ciphers", [], _tls_cipher_suites,
-                                length_from=lambda pkt: pkt.cipherslen),
-                    XStrLenField("connection_id", b"",
-                                length_from=lambda pkt: pkt.connection_idlen) ]
+                   _SSLv2CertDataField("cert", b"",
+                                       length_from=lambda pkt: pkt.certlen),
+                   _SSLv2CipherSuitesField("ciphers", [], _tls_cipher_suites,
+                                           length_from=lambda pkt: pkt.cipherslen),  # noqa: E501
+                   XStrLenField("connection_id", b"",
+                                length_from=lambda pkt: pkt.connection_idlen)]
 
     def tls_session_update(self, msg_str):
         """
@@ -195,7 +200,7 @@ class SSLv2ServerHello(_SSLv2Handshake):
 
 
 ###############################################################################
-### ClientMasterKey                                                         ###
+#   ClientMasterKey                                                           #
 ###############################################################################
 
 class _SSLv2CipherSuiteField(EnumField):
@@ -217,6 +222,7 @@ class _SSLv2CipherSuiteField(EnumField):
     def getfield(self, pkt, s):
         return s[3:], self.m2i(pkt, s)
 
+
 class _SSLv2EncryptedKeyField(XStrLenField):
     def i2repr(self, pkt, x):
         s = super(_SSLv2EncryptedKeyField, self).i2repr(pkt, x)
@@ -226,28 +232,29 @@ class _SSLv2EncryptedKeyField(XStrLenField):
             s += "    [decryptedkey= %s]" % ds
         return s
 
+
 class SSLv2ClientMasterKey(_SSLv2Handshake):
     """
     SSLv2 ClientMasterKey.
     """
     __slots__ = ["decryptedkey"]
     name = "SSLv2 Handshake - Client Master Key"
-    fields_desc = [ ByteEnumField("msgtype", 2, _sslv2_handshake_type),
-                    _SSLv2CipherSuiteField("cipher", None, _tls_cipher_suites),
+    fields_desc = [ByteEnumField("msgtype", 2, _sslv2_handshake_type),
+                   _SSLv2CipherSuiteField("cipher", None, _tls_cipher_suites),
 
-                    FieldLenField("clearkeylen", None, fmt="!H",
-                                  length_of="clearkey"),
-                    FieldLenField("encryptedkeylen", None, fmt="!H",
-                                  length_of="encryptedkey"),
-                    FieldLenField("keyarglen", None, fmt="!H",
-                                  length_of="keyarg"),
+                   FieldLenField("clearkeylen", None, fmt="!H",
+                                 length_of="clearkey"),
+                   FieldLenField("encryptedkeylen", None, fmt="!H",
+                                 length_of="encryptedkey"),
+                   FieldLenField("keyarglen", None, fmt="!H",
+                                 length_of="keyarg"),
 
-                    XStrLenField("clearkey", "",
+                   XStrLenField("clearkey", "",
                                 length_from=lambda pkt: pkt.clearkeylen),
-                    _SSLv2EncryptedKeyField("encryptedkey", "",
-                                length_from=lambda pkt: pkt.encryptedkeylen),
-                    XStrLenField("keyarg", "",
-                                length_from=lambda pkt: pkt.keyarglen) ]
+                   _SSLv2EncryptedKeyField("encryptedkey", "",
+                                           length_from=lambda pkt: pkt.encryptedkeylen),  # noqa: E501
+                   XStrLenField("keyarg", "",
+                                length_from=lambda pkt: pkt.keyarglen)]
 
     def __init__(self, *args, **kargs):
         """
@@ -265,10 +272,10 @@ class SSLv2ClientMasterKey(_SSLv2Handshake):
         clearkeylen = struct.unpack("!H", s[4:6])[0]
         encryptedkeylen = struct.unpack("!H", s[6:8])[0]
         encryptedkeystart = 10 + clearkeylen
-        encryptedkey = s[encryptedkeystart:encryptedkeystart+encryptedkeylen]
+        encryptedkey = s[encryptedkeystart:encryptedkeystart + encryptedkeylen]
         if self.tls_session.server_rsa_key:
             self.decryptedkey = \
-                    self.tls_session.server_rsa_key.decrypt(encryptedkey)
+                self.tls_session.server_rsa_key.decrypt(encryptedkey)
         else:
             self.decryptedkey = None
         return s
@@ -283,7 +290,7 @@ class SSLv2ClientMasterKey(_SSLv2Handshake):
                 cs_val = 0x0700c0
                 cipher = b"\x07\x00\xc0"
             else:
-                cs_val = cs_vals[0]         #XXX choose the best one
+                cs_val = cs_vals[0]  # XXX choose the best one
                 cipher = struct.pack(">BH", cs_val >> 16, cs_val & 0x00ffff)
             cs_cls = _tls_cipher_suites_cls[cs_val]
             self.cipher = cs_val
@@ -298,7 +305,7 @@ class SSLv2ClientMasterKey(_SSLv2Handshake):
 
         if cs_cls:
             if (self.encryptedkey == b"" and
-                len(self.tls_session.server_certs) > 0):
+                    len(self.tls_session.server_certs) > 0):
                 # else, the user is responsible for export slicing & encryption
                 key = randstring(cs_cls.cipher_alg.key_len)
 
@@ -332,9 +339,9 @@ class SSLv2ClientMasterKey(_SSLv2Handshake):
             self.keyarglen = len(keyarg)
         keyarglen = struct.pack("!H", self.keyarglen)
 
-        s = (chb(pkt[0]) + cipher
-             + clearkeylen + encryptedkeylen + keyarglen
-             + clearkey + encryptedkey + keyarg)
+        s = (pkt[:1] + cipher +
+             clearkeylen + encryptedkeylen + keyarglen +
+             clearkey + encryptedkey + keyarg)
         return s + pay
 
     def tls_session_update(self, msg_str):
@@ -352,14 +359,14 @@ class SSLv2ClientMasterKey(_SSLv2Handshake):
         connection_end = s.connection_end
         wcs_seq_num = s.wcs.seq_num
         s.pwcs = writeConnState(ciphersuite=cs_cls,
-                                            connection_end=connection_end,
-                                            seq_num=wcs_seq_num,
-                                            tls_version=tls_version)
+                                connection_end=connection_end,
+                                seq_num=wcs_seq_num,
+                                tls_version=tls_version)
         rcs_seq_num = s.rcs.seq_num
         s.prcs = readConnState(ciphersuite=cs_cls,
-                                           connection_end=connection_end,
-                                           seq_num=rcs_seq_num,
-                                           tls_version=tls_version)
+                               connection_end=connection_end,
+                               seq_num=rcs_seq_num,
+                               tls_version=tls_version)
 
         if self.decryptedkey is not None:
             s.master_secret = self.clearkey + self.decryptedkey
@@ -375,7 +382,7 @@ class SSLv2ClientMasterKey(_SSLv2Handshake):
 
 
 ###############################################################################
-### ServerVerify                                                            ###
+#   ServerVerify                                                              #
 ###############################################################################
 
 class SSLv2ServerVerify(_SSLv2Handshake):
@@ -384,8 +391,8 @@ class SSLv2ServerVerify(_SSLv2Handshake):
     fed to the class. This is how SSLv2 defines the challenge length...
     """
     name = "SSLv2 Handshake - Server Verify"
-    fields_desc = [ ByteEnumField("msgtype", 5, _sslv2_handshake_type),
-                    XStrField("challenge", "") ]
+    fields_desc = [ByteEnumField("msgtype", 5, _sslv2_handshake_type),
+                   XStrField("challenge", "")]
 
     def build(self, *args, **kargs):
         fval = self.getfieldval("challenge")
@@ -398,11 +405,11 @@ class SSLv2ServerVerify(_SSLv2Handshake):
         if s.sslv2_challenge is not None:
             if self.challenge != s.sslv2_challenge:
                 pkt_info = pkt.firstlayer().summary()
-                log_runtime.info("TLS: invalid ServerVerify received [%s]", pkt_info)
+                log_runtime.info("TLS: invalid ServerVerify received [%s]", pkt_info)  # noqa: E501
 
 
 ###############################################################################
-### RequestCertificate                                                      ###
+#   RequestCertificate                                                        #
 ###############################################################################
 
 class SSLv2RequestCertificate(_SSLv2Handshake):
@@ -411,9 +418,9 @@ class SSLv2RequestCertificate(_SSLv2Handshake):
     fed to the class. This is how SSLv2 defines the challenge length...
     """
     name = "SSLv2 Handshake - Request Certificate"
-    fields_desc = [ ByteEnumField("msgtype", 7, _sslv2_handshake_type),
-                    ByteEnumField("authtype", 1, {1: "md5_with_rsa"}),
-                    XStrField("challenge", "") ]
+    fields_desc = [ByteEnumField("msgtype", 7, _sslv2_handshake_type),
+                   ByteEnumField("authtype", 1, {1: "md5_with_rsa"}),
+                   XStrField("challenge", "")]
 
     def tls_session_update(self, msg_str):
         super(SSLv2RequestCertificate, self).tls_session_update(msg_str)
@@ -421,7 +428,7 @@ class SSLv2RequestCertificate(_SSLv2Handshake):
 
 
 ###############################################################################
-### ClientCertificate                                                       ###
+#   ClientCertificate                                                         #
 ###############################################################################
 
 class SSLv2ClientCertificate(_SSLv2Handshake):
@@ -429,18 +436,18 @@ class SSLv2ClientCertificate(_SSLv2Handshake):
     SSLv2 ClientCertificate.
     """
     name = "SSLv2 Handshake - Client Certificate"
-    fields_desc = [ ByteEnumField("msgtype", 8, _sslv2_handshake_type),
+    fields_desc = [ByteEnumField("msgtype", 8, _sslv2_handshake_type),
 
-                    ByteEnumField("certtype", 1, {1: "x509_cert"}),
-                    FieldLenField("certlen", None, fmt="!H",
-                                  length_of="certdata"),
-                    FieldLenField("responselen", None, fmt="!H",
-                                  length_of="responsedata"),
+                   ByteEnumField("certtype", 1, {1: "x509_cert"}),
+                   FieldLenField("certlen", None, fmt="!H",
+                                 length_of="certdata"),
+                   FieldLenField("responselen", None, fmt="!H",
+                                 length_of="responsedata"),
 
-                    _SSLv2CertDataField("certdata", b"",
-                                      length_from=lambda pkt: pkt.certlen),
-                    _TLSSignatureField("responsedata", None,
-                                length_from=lambda pkt: pkt.responselen) ]
+                   _SSLv2CertDataField("certdata", b"",
+                                       length_from=lambda pkt: pkt.certlen),
+                   _TLSSignatureField("responsedata", None,
+                                      length_from=lambda pkt: pkt.responselen)]
 
     def build(self, *args, **kargs):
         s = self.tls_session
@@ -475,7 +482,7 @@ class SSLv2ClientCertificate(_SSLv2Handshake):
             sig_test = self.responsedata._verify_sig(m, s.client_certs[0])
             if not sig_test:
                 pkt_info = self.firstlayer().summary()
-                log_runtime.info("TLS: invalid client CertificateVerify signature [%s]", pkt_info)
+                log_runtime.info("TLS: invalid client CertificateVerify signature [%s]", pkt_info)  # noqa: E501
 
     def tls_session_update(self, msg_str):
         super(SSLv2ClientCertificate, self).tls_session_update(msg_str)
@@ -484,7 +491,7 @@ class SSLv2ClientCertificate(_SSLv2Handshake):
 
 
 ###############################################################################
-### Finished                                                                ###
+#   Finished                                                                  #
 ###############################################################################
 
 class SSLv2ClientFinished(_SSLv2Handshake):
@@ -493,8 +500,8 @@ class SSLv2ClientFinished(_SSLv2Handshake):
     to the class. SSLv2 does not offer any other way to know the c_id length.
     """
     name = "SSLv2 Handshake - Client Finished"
-    fields_desc = [ ByteEnumField("msgtype", 3, _sslv2_handshake_type),
-                    XStrField("connection_id", "") ]
+    fields_desc = [ByteEnumField("msgtype", 3, _sslv2_handshake_type),
+                   XStrField("connection_id", "")]
 
     def build(self, *args, **kargs):
         fval = self.getfieldval("connection_id")
@@ -507,7 +514,7 @@ class SSLv2ClientFinished(_SSLv2Handshake):
         if s.sslv2_connection_id is not None:
             if self.connection_id != s.sslv2_connection_id:
                 pkt_info = pkt.firstlayer().summary()
-                log_runtime.info("TLS: invalid client Finished received [%s]", pkt_info)
+                log_runtime.info("TLS: invalid client Finished received [%s]", pkt_info)  # noqa: E501
 
 
 class SSLv2ServerFinished(_SSLv2Handshake):
@@ -516,8 +523,8 @@ class SSLv2ServerFinished(_SSLv2Handshake):
     to the class. SSLv2 does not offer any other way to know the sid length.
     """
     name = "SSLv2 Handshake - Server Finished"
-    fields_desc = [ ByteEnumField("msgtype", 6, _sslv2_handshake_type),
-                    XStrField("sid", "") ]
+    fields_desc = [ByteEnumField("msgtype", 6, _sslv2_handshake_type),
+                   XStrField("sid", "")]
 
     def build(self, *args, **kargs):
         fval = self.getfieldval("sid")
@@ -531,12 +538,11 @@ class SSLv2ServerFinished(_SSLv2Handshake):
 
 
 ###############################################################################
-### All handshake messages defined in this module                           ###
+#   All handshake messages defined in this module                             #
 ###############################################################################
 
-_sslv2_handshake_cls = { 0: SSLv2Error,             1: SSLv2ClientHello,
-                         2: SSLv2ClientMasterKey,   3: SSLv2ClientFinished,
-                         4: SSLv2ServerHello,       5: SSLv2ServerVerify,
-                         6: SSLv2ServerFinished,    7: SSLv2RequestCertificate,
-                         8: SSLv2ClientCertificate }
-
+_sslv2_handshake_cls = {0: SSLv2Error, 1: SSLv2ClientHello,
+                        2: SSLv2ClientMasterKey, 3: SSLv2ClientFinished,
+                        4: SSLv2ServerHello, 5: SSLv2ServerVerify,
+                        6: SSLv2ServerFinished, 7: SSLv2RequestCertificate,
+                        8: SSLv2ClientCertificate}
